@@ -635,7 +635,7 @@ static int handle_host_check( int event_type, void *data ) {
         chk_result->output_file         = 0;
         chk_result->output_file_fd      = -1;
         chk_result->output              = strdup(temp_buffer);
-        chk_result->return_code         = 2;
+        chk_result->return_code         = mod_gm_opt->orphan_return;
         chk_result->check_options       = CHECK_OPTION_NONE;
         chk_result->object_check_type   = HOST_CHECK;
         chk_result->check_type          = HOST_CHECK_ACTIVE;
@@ -838,8 +838,17 @@ static int read_arguments( const char *args_orig ) {
     int verify;
     int errors = 0;
     char *ptr;
-    char *args   = strdup(args_orig);
-    char *args_c = args;
+    char *args;
+    char *args_c;
+
+    if (args_orig == NULL) {
+        gm_log( GM_LOG_ERROR, "error parsing arguments: none provided.\n" );
+        return GM_ERROR;
+    }
+
+    args = strdup(args_orig);
+    args_c = args;
+
     while ( (ptr = strsep( &args, " " )) != NULL ) {
         if(parse_args_line(mod_gm_opt, ptr, 0) != GM_OK) {
             errors++;
