@@ -414,6 +414,11 @@ int set_worker( gearman_worker_st *w ) {
     return GM_OK;
 }
 
+/* called when worker runs into exit timeout */
+void exit_sighandler(int sig) {
+    gm_log( GM_LOG_TRACE, "exit_sighandler(%i)\n", sig );
+    _exit( EXIT_SUCCESS );
+}
 
 /* called when worker runs into idle timeout */
 void idle_sighandler(int sig) {
@@ -432,6 +437,10 @@ void set_state(int status) {
     if(worker_run_mode == GM_WORKER_STANDALONE)
         return;
 
+    /* give us 10 seconds to set state */
+    signal(SIGALRM, exit_sighandler);
+    alarm(10);
+
     /* Now we attach the segment to our data space. */
     if ((shm = shmat(shmid, NULL, 0)) == (int *) -1) {
         perror("shmat");
@@ -445,6 +454,8 @@ void set_state(int status) {
     if(status == GM_JOB_END) {
         shm[SHM_JOBS_DONE]++; /* increase jobs done */
 
+        shm[SHM_WORKER_LAST_CHECK] = (int)time(NULL); /* set last job date */
+
         /* status slot changed to -1 -> exit */
         if( shm[shm_index] == -1 ) {
             gm_log( GM_LOG_TRACE, "worker finished: %d\n", getpid() );
@@ -465,6 +476,8 @@ void set_state(int status) {
     if(shmdt(shm) < 0)
         perror("shmdt");
 
+    alarm(0);
+
     return;
 }
 
@@ -473,6 +486,10 @@ void set_state(int status) {
 void clean_worker_exit(int sig) {
     int *shm;
 
+    /* give us 30 seconds to stop */
+    signal(SIGALRM, exit_sighandler);
+    alarm(30);
+
     gm_log( GM_LOG_TRACE, "clean_worker_exit(%d)\n", sig);
 
     /* clear gearmans job, otherwise it would be retried and retried */
@@ -546,10 +563,15 @@ void *return_status( gearman_job_st *job, void *context, size_t *result_size, ge
     result = malloc(GM_BUFFERSIZE);
     *result_size = GM_BUFFERSIZE;
 
+    /* give us 10 seconds to get state */
+    signal(SIGALRM, exit_sighandler);
+    alarm(10);
+
     /* Now we attach the segment to our data space. */
     if ((shm = shmat(shmid, NULL, 0)) == (int *) -1) {
         perror("shmat");
         *result_size = 0;
+        alarm(0);
         return NULL;
     }
 
@@ -562,6 +584,8 @@ void *return_status( gearman_job_st *job, void *context, size_t *result_size, ge
     if(shmdt(shm) < 0)
         perror("shmdt");
 
+    alarm(0);
+
     return((void*)result);
 }
 
